@@ -13,7 +13,7 @@
 (require 'byte-compile "bytecomp")
 
 (if (eq byte-compile-warnings t)
-    (setq byte-compile-warnings byte-compile-warning-types))
+    (setq byte-compile-warnings byte-compile-default-warnings))
 
 (setq byte-compile-warnings
       (delq 'free-vars byte-compile-warnings))
