@@ -4,7 +4,7 @@
 
 ;; Author: Hans Henrik Eriksen <hhe@ifi.uio.no>
 ;; Maintainer: simula-mode@ifi.uio.no
-;; Version: 0.99
+;; Version: 0.992
 ;; Adapted-By: ESR
 ;; Keywords: languages
 
@@ -12,7 +12,7 @@
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 1, or (at your option)
+;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
@@ -26,7 +26,11 @@
 
 ;;; Commentary:
 
-;; Hans Henrik Eriksen may be reached at:
+;; A major mode for editing the Simula language.  It knows about Simula
+;; syntax and standard indentation commands.  It also provides convenient
+;; abbrevs for Simula keywords.
+;;
+;; Hans Henrik Eriksen (the author) may be reached at:
 ;;         Institutt for informatikk,
 ;;         Universitetet i Oslo
 
@@ -35,8 +39,9 @@
 (provide 'simula-mode)
 
 (defconst simula-tab-always-indent nil
-  "*Non-nil means TAB in SIMULA mode should always reindent the current line,
-regardless of where in the line point is when the TAB command is used.")
+  "*Non-nil means TAB in SIMULA mode should always reindent the current line.
+Otherwise TAB indents only when point is within
+the run of whitespace at the beginning of the line.")
 
 (defconst simula-indent-level 3
   "*Indentation of SIMULA statements with respect to containing block.")
@@ -51,39 +56,39 @@ will have the car of the list extra indentation with respect to
 the previous line of the statement.")
 
 (defconst simula-label-offset -4711
-  "*Offset of SIMULA label lines relative to usual indentation")
+  "*Offset of SIMULA label lines relative to usual indentation.")
 
 (defconst simula-if-indent '(0 . 0)
   "*Extra indentation of THEN and ELSE with respect to the starting IF.
-Value is a cons cell, the car is extra THEN indention and the cdr
-extra ELSE indention. IF after ELSE is indented as the starting IF.")
+Value is a cons cell, the car is extra THEN indentation and the cdr
+extra ELSE indentation. IF after ELSE is indented as the starting IF.")
 
 (defconst simula-inspect-indent '(0 . 0)
-  "*Extra indentation of WHEN and OTHERWISE with respect to the
-corresponding INSPECT. Value is a cons cell, the car is
-extra WHEN indention and the cdr extra OTHERWISE indention.")
+  "*Extra indentation of WHEN and OTHERWISE with respect to the INSPECT.
+Value is a cons cell, the car is extra WHEN indentation
+and the cdr extra OTHERWISE indentation.")
 
 (defconst simula-electric-indent nil
-  "*If this variable is non-nil, the simula-indent-line function
-will check the previous line to see if it has to be reindented.")
+  "*Non-nil means `simula-indent-line' function may reindent previous line.")
 
 (defconst simula-abbrev-keyword 'upcase
-  "*Determine how SIMULA keywords will be expanded. Value is one of
-the symbols upcase, downcase, capitalize, (as in) abbrev-table or
-nil if they should not be changed.")
+  "*Specify how to convert case for SIMULA keywords.
+Value is one of the symbols `upcase', `downcase', `capitalize',
+(as in) `abbrev-table' or nil if they should not be changed.")
 
 (defconst simula-abbrev-stdproc 'abbrev-table
-  "*Determine how standard SIMULA procedure and class names will be
-expanded. Value is one of the symbols upcase, downcase, capitalize,
-(as in) abbrev-table or nil if they should not be changed.")
+  "*Specify how to convert case for standard SIMULA procedure and class names.
+Value is one of the symbols `upcase', `downcase', `capitalize',
+(as in) `abbrev-table', or nil if they should not be changed.")
 
 (defvar simula-abbrev-file nil
-  "*File with abbrev definitions that are merged together with
-the standard abbrev definitions.  Please note that the standard
-definitions are required for simula-mode to function correctly.")
+  "*File with extra abbrev definitions for use in SIMULA mode.
+These are used together with the standard abbrev definitions for SIMULA.
+Please note that the standard definitions are required
+for SIMULA mode to function correctly.")
 
 (defvar simula-mode-syntax-table nil
-  "Syntax table in simula-mode buffers.")
+  "Syntax table in SIMULA mode buffers.")
 
 (if simula-mode-syntax-table
     ()
@@ -104,7 +109,7 @@ definitions are required for simula-mode to function correctly.")
   (modify-syntax-entry ?\} "."    simula-mode-syntax-table))
 
 (defvar simula-mode-map ()
-  "Keymap used in simula mode.")
+  "Keymap used in SIMULA mode.")
 
 (if simula-mode-map
     ()
@@ -120,7 +125,7 @@ definitions are required for simula-mode to function correctly.")
   (define-key simula-mode-map "\t"         'simula-indent-command))
 
 (defvar simula-mode-abbrev-table nil
-  "Abbrev table in simula-mode buffers")
+  "Abbrev table in SIMULA mode buffers")
 
 
 (defun simula-mode ()
@@ -136,36 +141,36 @@ Variables controlling indentation style:
     Extra indentation after DO, THEN, ELSE, WHEN and OTHERWISE.
  simula-continued-statement-offset 3
     Extra indentation for lines not starting a statement or substatement,
-    e.g. a nested FOR-loop. If value is a list, each line in a multipple-
+    e.g. a nested FOR-loop. If value is a list, each line in a multiple-
     line continued statement will have the car of the list extra indentation
     with respect to the previous line of the statement.
  simula-label-offset -4711
-    Offset of SIMULA label lines relative to usual indentation
+    Offset of SIMULA label lines relative to usual indentation.
  simula-if-indent '(0 . 0)
     Extra indentation of THEN and ELSE with respect to the starting IF.
-    Value is a cons cell, the car is extra THEN indention and the cdr
-    extra ELSE indention. IF after ELSE is indented as the starting IF.
+    Value is a cons cell, the car is extra THEN indentation and the cdr
+    extra ELSE indentation. IF after ELSE is indented as the starting IF.
  simula-inspect-indent '(0 . 0)
     Extra indentation of WHEN and OTHERWISE with respect to the
     corresponding INSPECT. Value is a cons cell, the car is
-    extra WHEN indention and the cdr extra OTHERWISE indention.
+    extra WHEN indentation and the cdr extra OTHERWISE indentation.
  simula-electric-indent nil
-    If this variable  non-nil value, simula-indent-line
+    If this variable is non-nil, `simula-indent-line'
     will check the previous line to see if it has to be reindented.
  simula-abbrev-keyword 'upcase
     Determine how SIMULA keywords will be expanded. Value is one of
-    the symbols upcase, downcase, capitalize, (as in) abbrev-table or
-    nil if they should not be changed.
+    the symbols `upcase', `downcase', `capitalize', (as in) `abbrev-table',
+    or nil if they should not be changed.
  simula-abbrev-stdproc 'abbrev-table
     Determine how standard SIMULA procedure and class names will be
-    expanded. Value is one of the symbols upcase, downcase, capitalize,
-    (as in) abbrev-table or nil if they should not be changed.
+    expanded. Value is one of the symbols `upcase', `downcase', `capitalize',
+    (as in) `abbrev-table', or nil if they should not be changed.
 
 Turning on SIMULA mode calls the value of the variable simula-mode-hook
 with no arguments, if that value is non-nil
 
 Warning: simula-mode-hook should not read in an abbrev file without calling
-the function simula-install-standard-abbrevs afterwards, preferrably not
+the function simula-install-standard-abbrevs afterwards, preferably not
 at all."
   (interactive)
   (kill-all-local-variables)
@@ -209,8 +214,8 @@ at all."
 
 
 (defun simula-indent-line ()
-  "Indent this line as SIMULA code.  If simula-electric-indent
-is non-nil, indent previous line if necessary."
+  "Indent this line as SIMULA code.
+If `simula-electric-indent' is non-nil, indent previous line if necessary."
   (let ((origin (- (point-max) (point)))
 	(indent (simula-calculate-indent))
 	(case-fold-search t))
@@ -245,7 +250,7 @@ is non-nil, indent previous line if necessary."
 
 (defun simula-indent-command (&optional whole-exp)
   "Indent current line as SIMULA code, or insert TAB character.
-If simula-tab-always-indent is non-nil, always indent current line.
+If `simula-tab-always-indent' is non-nil, always indent current line.
 Otherwise, indent only if point is before any non-whitespace
 character on the line.
 
@@ -294,8 +299,8 @@ The relative indentation among the lines of the statement are preserved."
 
 
 (defun simula-context ()
-  "Returns value according to position of point inside SIMULA text:
-    0    point inside COMMENT
+  "Returns value according to syntactic SIMULA context of point.
+    0    point inside COMMENT comment
     1    point on SIMULA-compiler directive line
     2    point inside END comment
     3    point inside string
@@ -393,7 +398,7 @@ The relative indentation among the lines of the statement are preserved."
 
 
 (defun simula-electric-label ()
-  "If this is a label that starts the line, reindent the line"
+  "If this is a label that starts the line, reindent the line."
   (interactive)
   (expand-abbrev)
   (insert ?:)
@@ -401,9 +406,13 @@ The relative indentation among the lines of the statement are preserved."
 	(case-fold-search t)
 	;; don't mix a label with an assignment operator := :-
 	;; therefore look at next typed character...
-	(next-char (event-to-character (setq unread-command-event
-                                             (next-command-event))))
-	(com-char last-command-char))
+	(next-char (if (fboundp 'next-command-event)
+                       (event-to-character (setq unread-command-event
+                                                 (next-command-event)))
+                       ;; RMSmacs
+                       (setq unread-command-events (list (read-event)))))
+	;(com-char last-command-char) -- unused
+        )
     (unwind-protect
 	;; Problem: find out if character just read is a command char
 	;; that would insert something after ':' making it a label.
@@ -435,7 +444,7 @@ The relative indentation among the lines of the statement are preserved."
 
 (defun simula-backward-up-level (count)
   "Move backward up COUNT block levels.
-If COUNT is negative, move forward up block level instead"
+If COUNT is negative, move forward up block level instead."
   (interactive "p")
   (let ((origin (point))
 	(case-fold-search t))
@@ -466,7 +475,7 @@ If COUNT is negative, move forward up block level instead"
 
 (defun simula-forward-down-level (count)
   "Move forward down COUNT block levels.
-If COUNT is negative, move backward down block level instead"
+If COUNT is negative, move backward down block level instead."
   (interactive "p")
   ;; When we search for a deeper block level, we must never
   ;; get out of the block where we started -> count >= start-count
@@ -503,7 +512,7 @@ If COUNT is negative, move backward down block level instead"
      
 (defun simula-previous-statement (count)
   "Move backward COUNT statements.
-If COUNT is negative, move forward instead (simula-next-statement)"
+If COUNT is negative, move forward instead."
   (interactive "p")
   (if (< count 0)
       (simula-next-statement (- count))
@@ -535,8 +544,8 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 
 
 (defun simula-next-statement (count)
-  "Move backward COUNT statements.
-If COUNT is negative, move forward instead (simula-next-statement)"
+  "Move forward COUNT statements.
+If COUNT is negative, move backward instead."
   (interactive "p")
   (if (< count 0)
       (simula-previous-statement (- count))
@@ -560,7 +569,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 
 
 (defun simula-skip-comment-backward ()
-  "Search towards bob to find first char that is outside a comment"
+  "Search towards bob to find first char that is outside a comment."
   (interactive)
   (catch 'simula-out
     (let (context)
@@ -599,7 +608,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 
 
 (defun simula-skip-comment-forward ()
-  "Search towards eob to find first char that is outside a comment"
+  "Search towards eob to find first char that is outside a comment."
   ;; this function assumes we start with point .outside a comment
   (interactive)
   (catch 'simula-out
@@ -677,7 +686,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
        ;; Calculate non-comment indentation
        (t
 	;; first, find out if this line starts with something that needs
-	;; special indention (END/IF/THEN/ELSE/WHEN/OTHERWISE or label)
+	;; special indentation (END/IF/THEN/ELSE/WHEN/OTHERWISE or label)
 	;;
 	(skip-chars-forward " \t\f")
 	(cond
@@ -780,7 +789,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 				    (cdr simula-if-indent))))
 		  (simula-find-inspect)))
 		;; found the start of a [sub]statement
-		;; add indention for continued statement
+		;; add indentation for continued statement
 		(if continued
 		    (setq indent
 			  (+ indent
@@ -818,7 +827,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 				   "begin\\|then\\|else\\|when\\|otherwise\\|do")))
 			 (not (memq (preceding-char) '(?: ?\;))))))
 	    ;; if we the state of the continued-variable
-	    ;; changed, add indention for continued statement
+	    ;; changed, add indentation for continued statement
 	    (if (or (and prev-cont (not continued))
 		    (and continued
 			 (listp simula-continued-statement-offset)))
@@ -832,13 +841,13 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 		(setq start-line (save-excursion (beginning-of-line) (point)))
 	      (beginning-of-line))))
         ;;
-	;; return indention
+	;; return indentation
 	;;
 	indent)))))
 
 
 (defun simula-find-if ()
-  "Find starting IF of a IF-THEN[-ELSE[-IF-THEN...]] statement"
+  "Find starting IF of a IF-THEN[-ELSE[-IF-THEN...]] statement."
   (catch 'simula-out
     (while t
       (if (and (simula-search-backward "\\<if\\>\\|;\\|\\<begin\\>"nil t)
@@ -863,7 +872,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 
 
 (defun simula-find-inspect ()
-  "Find INSPECT matching WHEN or OTHERWISE"
+  "Find INSPECT matching WHEN or OTHERWISE."
   (catch 'simula-out
     (let ((level 0))
       ;;
@@ -894,7 +903,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 
 
 (defun simula-inside-parens ()
-  "Return position after '(' on line if inside parentheses, nil otherwise."
+  "Return position after `(' on line if inside parentheses, nil otherwise."
   (save-excursion
     (let ((parlevel 0))
       (catch 'simula-out
@@ -906,7 +915,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 		   ((eq (following-char) ?\;)
 		    (if (zerop parlevel)
 			(throw 'simula-out nil)
-		      (error "Parethesis mismatch or misplaced ';'")))
+		      (error "Parenthesis mismatch or misplaced ';'")))
 		   ((eq (following-char) ?\()
 		    (if (zerop parlevel)
 			(throw 'simula-out (1+ (current-column)))
@@ -944,7 +953,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
   "Expand SIMULA keyword. If it starts the line, reindent."
   ;; redisplay
   (let ((show-char (eq this-command 'self-insert-command)))
-    ;; If the abbrev expansion results in reindention, the user may have
+    ;; If the abbrev expansion results in reindentation, the user may have
     ;; to wait some time before the character he typed is displayed
     ;; (the char causing the expansion is inserted AFTER the hook function
     ;; is called). This is annoying in case of normal characters.
@@ -970,8 +979,8 @@ If COUNT is negative, move forward instead (simula-next-statement)"
        ((eq simula-abbrev-keyword 'capitalize) (capitalize-word -1)))
       (let ((pos (- (point-max) (point)))
 	    (case-fold-search t)
-	    null)
-	(condition-case null
+	    )
+	(condition-case nil
 	    (progn
 	      ;; check if the expanded word is on the beginning of the line.
 	      (if (and (eq (char-syntax (preceding-char)) ?w)
@@ -1048,8 +1057,7 @@ If COUNT is negative, move forward instead (simula-next-statement)"
 
   
 (defun simula-install-standard-abbrevs ()
-  "Define Simula keywords, standard procedures and classes in
-local abbrev table."
+  "Define Simula keywords, procedures and classes in local abbrev table."
   ;; procedure and class names are as of the SIMULA 87 standard.
   (interactive)
   (mapcar (function (lambda (args)
