@@ -192,6 +192,14 @@ With prefix arg, prompt for diff switches."
       nil)))
   (setq new (expand-file-name new)
 	old (expand-file-name old))
+  ;; lemacs addition -- allow (diff "../old/" "new-file.el") to work
+  (cond ((file-directory-p old)
+         (or (file-directory-p new)
+             (setq old (expand-file-name (file-name-nondirectory new)
+                                         (file-name-as-directory old)))))
+        ((file-directory-p new)
+         (setq new (expand-file-name (file-name-nondirectory old)
+                                     (file-name-as-directory new)))))
   (let ((old-alt (file-local-copy old))
 	(new-alt (file-local-copy new))
 	buf)
