@@ -1,9 +1,10 @@
-;;;; $Id: compile-all.el,v 1.4 1992/10/29 02:43:41 jwz Exp $
-;;;; This file byte-compiles all .el files in pcl-cvs release 1.03.1.
+;;;; @(#) Id: compile-all.el,v 1.11 1993/05/31 18:40:25 ceder Exp 
+;;;; This file byte-compiles all .el files in pcl-cvs release 1.05.
 ;;;;
 ;;;; Copyright (C) 1991 Inge Wallin
 ;;;;
-;;;; This file is part of the GNU Emacs lisp library, Elib.
+;;;; This file was once upon a time part of Elib, but have since been
+;;;; modified by Per Cederqvist.
 ;;;;
 ;;;; GNU Elib is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -21,14 +22,11 @@
 ;;;;
 
 
-(setq elib-files '("elib-node"
-		   "elib-dll"
-		   "cookie"
-		   "pcl-cvs"))
+(setq files-to-compile '("pcl-cvs" "pcl-cvs-lucid"))
 
 
 (defun compile-file-if-necessary (file)
-  "Compile the Elib file FILE if necessary.
+  "Compile FILE if necessary.
 
 This is done if FILE.el is newer than FILE.elc or if FILE.elc doesn't exist."
   (let ((el-name (concat file ".el"))
@@ -41,12 +39,14 @@ This is done if FILE.el is newer than FILE.elc or if FILE.elc doesn't exist."
 
 
 (defun compile-pcl-cvs ()
-  "Byte-compile all uncompiled files of elib.
-Be sure to have . in load-path since a number of files in elib
-depend on other files and we always want the newer one even if
-a previous version of elib exists."
+  "Byte-compile all uncompiled files of pcl-cvs."
 
   (interactive)
-  (setq load-path (append '(".") load-path))
-  (mapcar (function compile-file-if-necessary)
-	  elib-files))
+
+  ;; Be sure to have . in load-path since a number of files
+  ;; depend on other files and we always want the newer one even if
+  ;; a previous version of pcl-cvs exists.
+  (let ((load-path (append '(".") load-path)))
+
+    (mapcar (function compile-file-if-necessary)
+	    files-to-compile)))
