@@ -1,5 +1,5 @@
 ;;; Commands to move around within a VM message
-;;; Copyright (C) 1989, 1990, 1991 Kyle E. Jones
+;;; Copyright (C) 1989, 1990, 1991, 1993, 1994 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -27,28 +27,53 @@ Prefix N scrolls forward N lines."
     (vm-select-folder-buffer)
     (vm-check-for-killed-summary)
     (vm-error-if-folder-empty)
-    (if (null (get-buffer-window (current-buffer)))
-	(progn
-	  (vm-display-current-message-buffer)
+    (if (null (vm-get-buffer-window (current-buffer)))
+	(let ((point (point)))
+	  (vm-display (current-buffer) t
+		      '(vm-scroll-forward vm-scroll-backward)
+		      (list this-command 'reading-message))
+	  ;; window start sticks to end of clip region when clip
+	  ;; region moves back past it in the buffer.  fix it.
+	  (let ((w (vm-get-buffer-window (current-buffer))))
+	    (if (= (window-start w) (point-max))
+		(set-window-start w (point-min))))
 	  (setq was-invisible t)))
-    (if (eq vm-system-state 'previewing)
-	(progn (vm-show-current-message)
-	       (vm-howl-if-eom))
-      (if (or mp-changed was-invisible)
-	  (vm-howl-if-eom)
-	(let ((vmp vm-message-pointer)
-	      (msg-buf (current-buffer))
-	      (h-diff 0)
-	      w old-w old-w-height result)
+    (if (or mp-changed was-invisible
+	    (and (eq vm-system-state 'previewing)
+		 (pos-visible-in-window-p
+		  (point-max)
+		  (vm-get-buffer-window (current-buffer)))))
+	(progn
+	  (if (not was-invisible)
+	      (let ((w (vm-get-buffer-window (current-buffer)))
+		    old-w-start)
+		(setq old-w-start (window-start w))
+		(vm-display nil nil '(vm-scroll-forward vm-scroll-backward)
+			    (list this-command 'reading-message))
+		(setq w (vm-get-buffer-window (current-buffer)))
+		(and w (set-window-start w old-w-start))))
+	  (if (eq vm-system-state 'previewing)
+	      (vm-show-current-message))
+	  (vm-howl-if-eom))
+      (let ((vmp vm-message-pointer)
+	    (msg-buf (current-buffer))
+	    (h-diff 0)
+	    w old-w old-w-height old-w-start result)
+	(if (eq vm-system-state 'previewing)
+	    (vm-show-current-message))
 	(setq vm-system-state 'reading)
-	(setq old-w (get-buffer-window msg-buf)
-	      old-w-height (window-height old-w))
-	(if (not (eq vm-window-configuration 'paging-message))
-	    (save-excursion (vm-set-window-configuration 'paging-message)))
-	(setq w (get-buffer-window msg-buf))
+	(setq old-w (vm-get-buffer-window msg-buf)
+	      old-w-height (window-height old-w)
+	      old-w-start (window-start old-w))
+	(vm-display nil nil '(vm-scroll-forward vm-scroll-backward)
+		    (list this-command 'reading-message))
+	(setq w (vm-get-buffer-window msg-buf))
 	(if (null w)
-	    (error "paging-message configuration hides the message buffer.")
+	    (error "current window configuration hides the message buffer.")
 	  (setq h-diff (- (window-height w) old-w-height)))
+	;; must restore this since it gets clobbered by window
+	;; teardown and rebuild done by the window config stuff.
+	(set-window-start w old-w-start)
 	(setq old-w (selected-window))
 	(unwind-protect
 	    (progn
@@ -69,15 +94,13 @@ Prefix N scrolls forward N lines."
 	  (select-window old-w))
 	(set-buffer msg-buf)
 	(cond ((eq result 'next-message)
-	       (vm-next-message)
-	       (vm-set-window-configuration 'auto-next-message))
+	       (vm-next-message))
 	      ((eq result 'end-of-message)
-	       (vm-set-window-configuration 'end-of-message)
 	       (let ((vm-message-pointer vmp))
 		 (vm-emit-eom-blurb)))
 	      (t
 	       (and (> (prefix-numeric-value arg) 0)
-		    (vm-howl-if-eom))))))))
+		    (vm-howl-if-eom)))))))
   (if (not (or vm-startup-message-displayed vm-inhibit-startup-message))
       (vm-display-startup-message)))
 
@@ -121,7 +144,7 @@ Prefix N scrolls forward N lines."
 ;; answer about where the end of the message is going to be
 ;; visible when redisplay finally does occur.
 (defun vm-howl-if-eom ()
-  (let ((w (get-buffer-window (current-buffer))))
+  (let ((w (vm-get-buffer-window (current-buffer))))
     (and w
 	 (save-excursion
 	   (save-window-excursion
@@ -130,16 +153,25 @@ Prefix N scrolls forward N lines."
 		   (select-window w)
 		   (save-excursion
 		     (save-window-excursion
-		       (scroll-up nil)))
+		       ;; scroll-fix.el replaces scroll-up and
+		       ;; doesn't behave properly when it hits
+		       ;; end of buffer.  It does this!
+		       ;; (ding)
+		       ;; (message (get 'beginning-of-buffer 'error-message))
+		       (let ((scroll-in-place-replace-original nil))
+			 (scroll-up nil))))
 		   nil)
 	       (error t))))
 	 (= (vm-text-end-of (car vm-message-pointer)) (point-max))
 	 (vm-emit-eom-blurb))))
 
 (defun vm-emit-eom-blurb ()
-  (message "End of message %s from %s"
-	   (vm-number-of (car vm-message-pointer))
-	   (vm-su-full-name (car vm-message-pointer))))
+  (if (vm-full-name-of (car vm-message-pointer))
+      (message "End of message %s from %s"
+	       (vm-number-of (car vm-message-pointer))
+	       (vm-full-name-of (car vm-message-pointer)))
+    (message "End of message %s"
+	     (vm-number-of (car vm-message-pointer)))))
 
 (defun vm-scroll-backward (arg)
   "Scroll backward a screenful of text.
@@ -151,6 +183,67 @@ Prefix N scrolls backward N lines."
 			   ((symbolp arg) nil)
 			   (t arg))))
 
+;; for lucid emacs
+(defvar highlight-headers-regexp)
+
+(defun vm-highlight-headers ()
+  (cond
+   ((vm-lucid-emacs-p)
+    (require 'highlight-headers)
+    (let ((highlight-headers-regexp (or vm-highlighted-header-regexp
+					highlight-headers-regexp)))
+      (highlight-headers
+       ;; If you just use point-min - point-max, it's not able to
+       ;; hack citations or the signature. -jwz
+;;       (point-min) (point-max)
+       (vm-start-of (car vm-message-pointer))
+       (vm-end-of (car vm-message-pointer))
+       t)
+      ))
+   ((fboundp 'overlay-put)
+    (let (o-lists p)
+      (setq o-lists (overlay-lists)
+	    p (car o-lists))
+      (while p
+	(and (overlay-get (car p) 'vm-highlight)
+	     (delete-overlay (car p)))
+	(setq p (cdr p)))
+      (setq p (cdr o-lists))
+      (while p
+	(and (overlay-get (car p) 'vm-highlight)
+	     (delete-overlay (car p)))
+	(setq p (cdr p)))
+      (goto-char (point-min))
+      (while (vm-match-header)
+	(cond ((vm-match-header vm-highlighted-header-regexp)
+	       (setq p (make-overlay (vm-matched-header-contents-start)
+				     (vm-matched-header-contents-end)))
+	       (overlay-put p 'face vm-highlighted-header-face)
+	       (overlay-put p 'vm-highlight t)))
+	(goto-char (vm-matched-header-end)))))))
+
+(defun vm-unhighlight-region (start end)
+  (cond ((vm-lucid-emacs-p)
+	 ;; The Lucid highlighting isn't set up to travel with the text.
+	 nil)
+	((fboundp 'put-text-property)
+	 ;; jwz: I think that nil would be better than `default' here,
+	 ;; since that means "unspecified".  In lemacs at least, that
+	 ;; would make a difference (who knows what rms did.)
+	 (put-text-property start end 'face 'default))))
+
+(defun vm-highlight-region (start end face)
+  (cond ((vm-lucid-emacs-p)
+	 ;; Need to think about this some more for lemacs; do we want to
+	 ;; use text properties for this?  Since header highlighting isn't
+	 ;; done with text properties, doing the summary with them could
+	 ;; be tricky.  I think that a higher level interface like
+	 ;; vm-[un]highlight-summary-line would be the best place for a
+	 ;; lemacs-specific implementation of this.
+	 nil)
+	((fboundp 'put-text-property)
+	 (put-text-property start end 'face face))))
+
 (defun vm-preview-current-message ()
   (setq vm-system-state 'previewing)
   (if vm-real-buffers
@@ -167,10 +260,8 @@ Prefix N scrolls backward N lines."
 	     (forward-line (if (natnump vm-preview-lines) vm-preview-lines 0))
 	     (point))))
 	  (t (vm-text-end-of (car vm-message-pointer)))))
-  ;; run the hooks here instead of before the narrow-to-region
-  ;; for Jamie Z's highlighting hooks.  reordering the headers
-  ;; screws up the extents.  the reference to vm-vheaders-of above
-  ;; assures that the rordering will have occurred by this point.
+  (vm-highlight-headers)
+
   (vm-run-message-hook (car vm-message-pointer) 'vm-select-message-hook)
   (and vm-select-new-message-hook (vm-new-flag (car vm-message-pointer))
        (vm-run-message-hook (car vm-message-pointer)
@@ -178,11 +269,12 @@ Prefix N scrolls backward N lines."
   (and vm-select-unread-message-hook (vm-unread-flag (car vm-message-pointer))
        (vm-run-message-hook (car vm-message-pointer)
 			    'vm-select-unread-message-hook))
+
   (if vm-honor-page-delimiters
       (vm-narrow-to-page))
   (goto-char (vm-text-of (car vm-message-pointer)))
   ;; If we have a window, set window start appropriately.
-  (let ((w (get-buffer-window (current-buffer))))
+  (let ((w (vm-get-buffer-window (current-buffer))))
     (if w
 	(progn (set-window-start w (point-min))
 	       (set-window-point w (vm-text-of (car vm-message-pointer))))))
@@ -205,12 +297,12 @@ Prefix N scrolls backward N lines."
 	      (forward-page 1))
 	  (vm-narrow-to-page))))
   ;; don't mark the message as read if the user can't see it!
-  (if (get-buffer-window (current-buffer))
+  (if (vm-get-buffer-window (current-buffer))
       (progn
 	(setq vm-system-state 'showing)
 	(cond ((vm-new-flag (car vm-message-pointer))
-	       (vm-set-new-flag (car vm-message-pointer) nil))
-	      ((vm-unread-flag (car vm-message-pointer))
+	       (vm-set-new-flag (car vm-message-pointer) nil)))
+	(cond ((vm-unread-flag (car vm-message-pointer))
 	       (vm-set-unread-flag (car vm-message-pointer) nil)))
 	(vm-update-summary-and-mode-line)
 	(vm-howl-if-eom))
@@ -223,6 +315,8 @@ Prefix N scrolls backward N lines."
   (vm-select-folder-buffer)
   (vm-check-for-killed-summary)
   (vm-error-if-folder-empty)
+  (vm-display (current-buffer) t '(vm-expose-hidden-headers)
+	      '(vm-expose-hidden-headers reading-message))
   (let* ((exposed (= (point-min) (vm-start-of (car vm-message-pointer)))))
     (vm-widen-page)
     (goto-char (point-max))
@@ -232,7 +326,7 @@ Prefix N scrolls backward N lines."
       (narrow-to-region (point) (vm-start-of (car vm-message-pointer))))
     (goto-char (point-min))
     (let (w)
-      (setq w (get-buffer-window (current-buffer)))
+      (setq w (vm-get-buffer-window (current-buffer)))
       (and w (set-window-point w (point-min)))
       (and w
 	   (= (window-start w) (vm-vheaders-of (car vm-message-pointer)))
@@ -272,10 +366,17 @@ Prefix N scrolls backward N lines."
   (vm-error-if-folder-empty)
   (vm-widen-page)
   (push-mark)
-  (goto-char (point-min))
+  (vm-display (current-buffer) t '(vm-beginning-of-message)
+	      '(vm-beginning-of-message reading-message))
+  (let ((osw (selected-window)))
+    (unwind-protect
+	(progn
+	  (select-window (vm-get-buffer-window (current-buffer)))
+	  (goto-char (point-min)))
+      (if (not (eq osw (selected-window)))
+	  (select-window osw))))
   (if vm-honor-page-delimiters
-      (vm-narrow-to-page))
-  (vm-display-current-message-buffer))
+      (vm-narrow-to-page)))
 
 (defun vm-end-of-message ()
   "Moves to the end of the current message, exposing and flagging it read
@@ -290,7 +391,14 @@ as necessary."
   (setq vm-system-state 'reading)
   (vm-widen-page)
   (push-mark)
-  (goto-char (point-max))
+  (vm-display (current-buffer) t '(vm-end-of-message)
+	      '(vm-end-of-message reading-message))
+  (let ((osw (selected-window)))
+    (unwind-protect
+	(progn
+	  (select-window (vm-get-buffer-window (current-buffer)))
+	  (goto-char (point-max)))
+      (if (not (eq osw (selected-window)))
+	  (select-window osw))))
   (if vm-honor-page-delimiters
-      (vm-narrow-to-page))
-  (vm-display-current-message-buffer))
+      (vm-narrow-to-page)))
