@@ -1,11 +1,11 @@
-/* Definitions file for GNU Emacs running on bsd 4.3
+/* Definitions file for GNU Emacs running on bsd 4.2
    Copyright (C) 1985, 1986 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -23,14 +23,12 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  *	Define all the symbols that apply correctly.
  */
 
-/* We give these symbols the numeric values found in <sys/param.h> to
-   avoid warnings about redefined macros.  */
-#ifndef BSD4_3
-#define BSD4_3 1
-#endif /* BSD4_3 */
+#ifndef BSD4_2
+#define BSD4_2
+#endif /* BSD4_2 */
 
 #ifndef BSD
-#define BSD 43
+#define BSD
 #endif /* BSD */
 
 /* SYSTEM_TYPE should indicate the kind of system you are using.
@@ -44,10 +42,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* #define NOMULTIPLEJOBS */
 
-/* Do not use interrupt_input = 1 by default, because in 4.3
-   we can make noninterrupt input work properly.  */
+/* Default is to set interrupt_input to 1: do input buffering within Emacs */
 
-#undef INTERRUPT_INPUT
+#define INTERRUPT_INPUT
 
 /* First pty name is /dev/ptyp0.  */
 
@@ -55,11 +52,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
  *	Define HAVE_TIMEVAL if the system supports the BSD style clock values.
- *	Define HAVE_SYS_TIME_H to include <sys/time.h> for a timeval structure.
+ *	Look in <sys/time.h> for a timeval structure.
  */
 
 #define HAVE_TIMEVAL
-#define HAVE_SYS_TIME_H
  
 /*
  *	Define HAVE_SELECT if the system supports the `select' system call.
@@ -81,10 +77,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define HAVE_SOCKETS
 
-/* Define HAVE_RENAME if system provides rename(2) */
-
-#define HAVE_RENAME
-
 /*
  *	Define NONSYSTEM_DIR_LIBRARY to make Emacs emulate
  *      The 4.2 opendir, etc., library functions.
@@ -114,7 +106,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    The alternative is that a lock file named
    /usr/spool/mail/$USER.lock.  */
 
-#define MAIL_USE_FLOCK
+/* #define MAIL_USE_FLOCK */
 
 /* Define CLASH_DETECTION if you want lock files to be written
    so that Emacs can tell instantly when you try to modify
@@ -138,3 +130,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* Send signals to subprocesses by "typing" special chars at them.  */
 
 #define SIGNALS_VIA_CHARACTERS
+
+/* We do have vfork.  */
+
+#define HAVE_VFORK
+
+/* Process groups work in the traditional BSD manner.  */
+
+#define BSD_PGRPS
