@@ -1,11 +1,11 @@
-/* Definitions file for GNU Emacs running on Silicon Graphics system 3.6.
+/* Definitions file for GNU Emacs running on Silicon Graphics 3.5
    Copyright (C) 1987 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -141,10 +141,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  */
 
 #define sigsetmask(mask)	/* Null expansion */
-#define sigblock(x) x
 
 /* The IRIS defines SIGIO in signal.h, but doesn't implement it. */
-/* #undef SIGIO		use BROKEN_SIGIO instead of this kludge -- jwz */
 #define BROKEN_SIGIO
 
 #define LIBS_MACHINE -lbsd -ldbm -lPW
@@ -156,14 +154,12 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define _setjmp setjmp
 #define _longjmp longjmp
 
-/* On USG systems the system calls are interruptable by signals
+/* On USG systems the system calls are interruptible by signals
  that the user program has elected to catch.  Thus the system call
- must be retried in these cases.  All calls to read, write, and open
- in emacs are really calls to emacs_read, etc.  We define emacs_read
- to be sys_read (which is defined in sysdep.c for this system.)  If
- these were not defined, they would be defined to be open, etc.
- We can't just "#define open sys_open" because of prototype problems.
- */
+ must be retried in these cases.  To handle this without massive
+ changes in the source code, we remap the standard system call names
+ to names for our own functions in sysdep.c that do the system call
+ with retries. */
 
 #define emacs_read sys_read
 #define emacs_open sys_open
@@ -191,9 +187,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define NEED_ERRNO
 
+/* The symbol FIONREAD is defined, but the feature does not work.  */
+
+#define BROKEN_FIONREAD
+
 /* This is how to get the device name of the tty end of a pty.  */
 #define PTY_TTY_NAME_SPRINTF \
  	    sprintf (ptyname, "/dev/ttyq%d", minor (stb.st_rdev));
-
-/* The C library does have the getwd function.  */
-#define HAVE_GETWD
