@@ -1,11 +1,11 @@
-/* Definitions file for GNU Emacs running on AT&T's System V Release 3
-   Copyright (C) 1987 Free Software Foundation, Inc.
+/* Definitions file for GNU Emacs running on AT&T's System V Release 2.0
+   Copyright (C) 1985, 1986 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -27,8 +27,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define USG5
 
-#define USG5_3
-
 /* SYSTEM_TYPE should indicate the kind of system you are using.
  It sets the Lisp variable system-type.  */
 
@@ -67,18 +65,13 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  *	Define HAVE_SELECT if the system supports the `select' system call.
  */
 
-/* SVr3.2 ports of X include an emulation of select in the libraries.  */
-#ifdef HAVE_X_WINDOWS
-#define HAVE_SELECT
-#endif
+/* #define HAVE_SELECT */
 
 /*
  *	Define HAVE_PTYS if the system supports pty devices.
  */
 
-/* Some versions of V.3 have this, but not all.
-   #define HAVE_PTYS
-   #define SYSV_PTYS  */
+/* #define HAVE_PTYS */
 
 /* Define HAVE_SOCKETS if system supports 4.2-compatible sockets.  */
 
@@ -89,14 +82,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  *      The 4.2 opendir, etc., library functions.
  */
 
-/* #define NONSYSTEM_DIR_LIBRARY */
-
-/*
- * 	Define SYSV_SYSTEM_DIR to use the V.3 getdents/readir
- *	library functions.  Almost, but not quite the same as
- *	the 4.2 functions
- */
-#define SYSV_SYSTEM_DIR
+#define NONSYSTEM_DIR_LIBRARY
 
 /* Define this symbol if your system has the functions bcopy, etc. */
 
@@ -133,9 +119,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* #define SHORTNAMES */
 
-/* We use the Berkeley (and usg5.2.2) interface to nlist.  */
+/* We do NOT use the Berkeley (and usg5.2.2) interface to nlist.  */
 
-#define NLIST_STRUCT
+/* #define NLIST_STRUCT */
 
 /* The file containing the kernel's symbol table is called /unix.  */
 
@@ -145,10 +131,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    is named avenrun.  */
 
 #define LDAV_SYMBOL "avenrun"
-
-/* Define this if system V IPC is available.  */
-
-#define HAVE_SYSVIPC
 
 /* Special hacks needed to make Emacs run on this system.  */
 
@@ -166,24 +148,22 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define _setjmp setjmp
 #define _longjmp longjmp
 
-/* On USG systems the system calls are interruptable by signals
+/* On USG systems the system calls are interruptible by signals
  that the user program has elected to catch.  Thus the system call
- must be retried in these cases.  All calls to read, write, and open
- in emacs are really calls to emacs_read, etc.  We define emacs_read
- to be sys_read (which is defined in sysdep.c for this system.)  If
- these were not defined, they would be defined to be open, etc.
- We can't just "#define open sys_open" because of prototype problems.
- */
+ must be retried in these cases.  To handle this without massive
+ changes in the source code, we remap the standard system call names
+ to names for our own functions in sysdep.c that do the system call
+ with retries. */
+
 #define emacs_read sys_read
-#define emacs_write sys_write
 #define emacs_open sys_open
-#define emacs_close sys_close
+#define emacs_write sys_write
 
 #define INTERRUPTIBLE_OPEN
-#define INTERRUPTIBLE_CLOSE
 #define INTERRUPTIBLE_IO
 
 /* On USG systems these have different names */
+
 #define index strchr
 #define rindex strrchr
 
@@ -197,48 +177,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define ADDR_CORRECT(x) (x)
 
+/* Prevent -lg from being used for debugging.  Not implemented?  */
+
+#define LIBS_DEBUG
+
 /* Use terminfo instead of termcap.  */
 
 #define TERMINFO
-
-/* Some variants have TIOCGETC, but the structures to go with it
-   are not declared.  */
-
-#define BROKEN_TIOCGETC
-
-/* AT&T SVr3 X wants to be linked with shared libraries */
-
-#define LIB_X11_LIB -lX11_s
-
-/* X needs to talk on the network, so search the network library.  */
-
-#define LIBX10_SYSTEM -lnsl_s
-#define LIBX11_SYSTEM -lnsl -lnsl_s -lpt -lc_s
-
-/* The docs for system V/386 suggest v.3 has sigpause,
-   so let's give it a try.  */
-#define HAVE_SYSV_SIGPAUSE
-
-/* Some variants have TIOCGWINSZ, but the structures to go with it
-   are not declared.  */
-
-#define BROKEN_TIOCGWINSZ
-
-/* SVr3 does not have utimes(2) */
-
-#define USE_UTIME
-
-/* If we're using the System V X port, BSD bstring functions will be handy */
-
-#ifdef HAVE_X_WINDOWS
-#define BSTRING
-#endif /* HAVE_X_WINDOWS */
-
-/* Enable support for shared libraries in unexec.  */
-
-#define USG_SHARED_LIBRARIES
-
-/* On USG systems signal handlers return void */
-
-#define SIGTYPE void
-#define SIGRETURN return
